﻿using System;
using System.Collections.Generic;
using WarpWorld.CrowdControl;

namespace TestEffectPack
{
    public class Card
    {
        public static Dictionary<CardID, UNO.FUNCTIONAL> Functionalities = new Dictionary<CardID, UNO.FUNCTIONAL>()
        {
            { CardID.None, UNO.FUNCTIONAL.normal },
            { CardID.Zero, UNO.FUNCTIONAL.zero },
            { CardID.One, UNO.FUNCTIONAL.normal },
            { CardID.Two, UNO.FUNCTIONAL.normal },
            { CardID.Three, UNO.FUNCTIONAL.normal },
            { CardID.Four, UNO.FUNCTIONAL.normal },
            { CardID.Five, UNO.FUNCTIONAL.normal },
            { CardID.Six, UNO.FUNCTIONAL.normal },
            { CardID.Seven, UNO.FUNCTIONAL.seven },
            { CardID.Eight, UNO.FUNCTIONAL.normal },
            { CardID.Nine, UNO.FUNCTIONAL.normal },
            { CardID.Skip, UNO.FUNCTIONAL.skip },
            { CardID.Reverse, UNO.FUNCTIONAL.reverse },
            { CardID.DrawTwo, UNO.FUNCTIONAL.draw2 },
            { CardID.Wild, UNO.FUNCTIONAL.wild },
            { CardID.Joker, UNO.FUNCTIONAL.joker },
        };

        public UNO.COLORS Color { get; private set; }
        public CardID Value { get; private set; }
        public UNO.FUNCTIONAL Functionality { get; private set; }

        public static Card GetRandom()
        {
            return new Card((UNO.COLORS)Rand.Next(0, 4), (CardID)Rand.Next(1, 15));
        }

        public Card(UNO.COLORS color, CardID value)
        {
            Color = color;
            Value = value;
            Functionality = Functionalities[Value];
        }

        public Card(UNO.Card unoCard)
        {
            Color = unoCard.Def.Color;
            Value = (CardID)unoCard.Def.Value;
            Functionality = Functionalities[Value];
        }

        public Card(string color, string value) {
            if (string.Equals(color, "Anything")) {
                Color = UNO.COLORS.NOColor;
            }
            else {
                Color = (UNO.COLORS)Enum.Parse(typeof(UNO.COLORS), color.ToLower());
            }

            if (string.Equals(value, "Anything"))
            {
                Value = CardID.None;
            }
            else {
                Value = (CardID)Enum.Parse(typeof(CardID), value);
            }

            Functionality = Functionalities[Value]; 
        }

        public static bool operator ==(Card card1, UNO.Card card2)
        {
            Card compareCard = new Card(card2.Def.Color, (CardID)card2.Def.Value);

            CrowdControl.LogError("COMPARING  "+ card1.Color + " " + card1.Value +" TO " + compareCard.Color + "-" + compareCard.Value);

            return card1 == compareCard;
        }

        public static bool operator !=(Card card1, UNO.Card card2) => !(card1 == card2);

        public static bool operator ==(Card card1, Card card2) {

            if (card1.Value == CardID.Wild && card2.Value == CardID.Wild) {
                return true;
            }

            if (card1.Value == CardID.Joker && card2.Value == CardID.Joker) {
                return true;
            }

            if (card1.Color != card2.Color && card1.Color != UNO.COLORS.NOColor && card2.Color != UNO.COLORS.NOColor) {
                return false;
            }

            if (card1.Value != card2.Value && card1.Value != CardID.None && card2.Value != CardID.None) {
                return false;
            }

            return true;
        }

        public static bool operator !=(Card card1, Card card2) => !(card1 == card2);

        public void Replace(UNO.Card gameCard) {
            gameCard.Def.Color = Color;
            gameCard.Def.Value = Convert.ToByte(Value);
            gameCard.Def.Functional = Data.CardFunctionalities[Value];
            gameCard.UpdateSurface();
        }
    }
}
