﻿using System;
using System.Collections.Generic;
using System.Timers;

namespace TestEffectPack
{
    public class Game
    {
        private static int MaxCards = 30;

        public static UNO.Player Player(string name) {
            PlayerTarget target = PlayerTarget.South;

            switch (name) {
                case "Bottom":
                    target = PlayerTarget.South;
                    break;
                case "Right":
                    target = PlayerTarget.East;
                    break;
                case "Top":
                    target = PlayerTarget.North;
                    break;
                case "Left":
                    target = PlayerTarget.West;
                    break;
            }

            return Player(target);
        }

        public static UNO.Player Player(PlayerTarget target)
        {
            return UNO.GameManager.GetInstance().players[Convert.ToByte(target)];
        }

        public static void UpdatePlayer(UNO.Player player, PlayerTarget target)
        {
            UNO.GameManager.GetInstance().players[Convert.ToByte(target)] = player;
        }

        public static bool DrawCardFromDeck(string side, Card card, int amount = 1)
        {
            UNO.Player player = Player(side);

            if (player.cards.Count + amount >= MaxCards) {
                return false;
            }

            for (int i = 0; i < amount; i++) {
                UNO.CheatManager.Instance.DrawOneCard(player);
                ReplaceCardAtIndex(player, card, player.cards.Count - 1);
            }

            player.UpdateCardsPosition();
            return true;
        }

        private static void DeletePlayerCard(UNO.Player player, int index)
        {
            DeletePlayerCard(player, player.cards[index]);
        }

        private static void DeletePlayerCard(UNO.Player player, UNO.Card card)
        {
            if (player.currentCard == card)
            {
                card.unselectMe();
            }

            player.cards.Remove(card);
            card.processer.RemoveCard(card);
            card.Destroy();

            player.UpdateCardsPosition();
        }

        public static bool RemoveRandomCards(UNO.Player player, int count = 1)
        {
            if (player.cards.Count < 2)
            {
                return false;
            }

            for (int i = 0; i < count && player.cards.Count > 0; i++)
            {
                DeletePlayerCard(player, Rand.Next(0, player.cards.Count));
            }

            return true;
        }

        public static bool RemoveCardsOfType(string side, Card cardToRemove, int count = 1) {
            UNO.Player player = Game.Player(side);

            List<UNO.Card> matchedCards = new List<UNO.Card>();

            foreach (UNO.Card card in player.cards) {
                if (cardToRemove == card) {
                    matchedCards.Add(card);
                }
            }

            if (matchedCards.Count < count) {
                return false;
            }

            for (int i = 0; i < count; i++) {
                int cardIndex = Rand.Next(0, matchedCards.Count);
                DeletePlayerCard(player, matchedCards[i]);
                matchedCards.RemoveAt(cardIndex);
            }

            player.UpdateCardsPosition();

            return true;
        }

        public static bool RemoveCardAtIndex(UNO.Player player, int index)
        {
            index--;

            if (player.cards.Count < index)
            {
                return false;
            }

            DeletePlayerCard(player, index);
            return true;
        }

        public static bool SetForUno(string side) {
            UNO.Player player = Game.Player(side);
            bool OK = RemoveRandomCards(player, player.cards.Count - 1);

            if (OK) {
                player.CallUno();
            }

            return OK;
        }

        ///

        public static bool ReplaceRandomCards(UNO.Player player, Card newCard, int count = 1)
        {
            if (player.cards.Count < count)
            {
                return false;
            }

            List<int> usedIndexes = new List<int>();

            for (int i = 0; i < count && player.cards.Count > 0; i++)
            {
                int randomIndex = Rand.Next(0, player.cards.Count);

                if (usedIndexes.Contains(randomIndex))
                {
                    i--;
                    continue;
                }

                newCard.Replace(player.cards[randomIndex]);
                usedIndexes.Add(randomIndex);
            }

            return true;
        }

        public static bool ReplaceCardsOfType(string side, Card oldCard, Card newCard, int count = 1) {
            UNO.Player player = Game.Player(side);
            List<UNO.Card> matchedCards = new List<UNO.Card>();

            foreach (UNO.Card card in player.cards) {
                if (oldCard == card) {
                    matchedCards.Add(card);
                }
            }

            if (matchedCards.Count < count) {
                return false;
            }

            for (int i = 0; i < count; i++) {
                int cardIndex = Rand.Next(0, matchedCards.Count);
                newCard.Replace(matchedCards[cardIndex]);
                matchedCards.RemoveAt(cardIndex);
            }

            return true;
        }

        public static bool ReplaceCardAtIndex(UNO.Player player, Card card, int index)
        {
            if (player.cards.Count < index)
            {
                return false;
            }

            card.Replace(player.cards[index]);
            return true;
        }

        public static bool ReplaceAllPlayerCards(UNO.Player player)
        {
            foreach (UNO.Card card in player.cards)
            {
                Card replacementCard = Card.GetRandom();
                replacementCard.Replace(card);
            }

            return true;
        }

        public static bool DrawTwoCards(UNO.Player player)
        {
            player.updateStatus(UNO.PLAYER_STATUS.draw2);
            //player.PlayerStatus = UNO.Player.EPlayerStatus.ePSDrawing2Cards;
            player.PunishDrawCards(false);
            return true;
        }

        private static bool SwitchPlayerState(UNO.Player player, string newState)
        {
            if (string.Equals(newState, player.GetCurrentStateName()))
            {
                return false;
            }

            player.SwitchPlayerState(newState);

            return true;
        }

        public static bool Reverse()
        {
            UNO.TurnManager.Instance.turnDirection = -1 * UNO.TurnManager.Instance.turnDirection;
            UNO.TurnManager.Instance.reverseTurnDirection();

            return true;
        }

        private static UNO.Deck_Enum GetDeck(string name) {
            switch (name) {
                case "Bottom":
                    return UNO.Deck_Enum.bottom;
                case "Right":
                    return UNO.Deck_Enum.right;
                case "Left":
                    return UNO.Deck_Enum.left;
            }

            return UNO.Deck_Enum.top;
        }

        public static bool SwapHand(string source, string target) {
            UNO.Deck_Enum sourceDeck = GetDeck(source);
            UNO.Deck_Enum targetDeck = GetDeck(target);

            if (sourceDeck == targetDeck) {
                return false;
            }

            UNO.Player player = Player(source);
            player.SwapHand(targetDeck);

            return true;
        }

        private static UNO.Player.StartTurnProcess[] skipEvents = new UNO.Player.StartTurnProcess[4];
        private static UNO.Player.StartTurnProcess[] draw2Events = new UNO.Player.StartTurnProcess[4];
        private static UNO.Player.StartTurnProcess[] draw4Events = new UNO.Player.StartTurnProcess[4];

        public static bool SkipPlayer(string target) {
            UNO.Player player = Player(target);

            if (skipEvents[player.Index] == null) {
                skipEvents[player.Index] = delegate () { SkipStartOfTurn(player, true); };
            }

            if (player.status != UNO.PLAYER_STATUS.outTurn) {
                return false;
            }

            player.onStartTurn += skipEvents[player.Index];
            return true;
        }
        
        private static void SkipStartOfTurn(UNO.Player player, bool eventCall) {
            if (player.status == UNO.PLAYER_STATUS.forbid || player.status == UNO.PLAYER_STATUS.draw2 || player.status == UNO.PLAYER_STATUS.draw4) {
                return;
            }

            player.ShowSkipEffect();
            player.ForceEndTurn();
            player.updateStatus(UNO.PLAYER_STATUS.forbid);
            UNO.TurnManager.Instance.UpdateTurnStatus(UNO.TurnStatusType.eIN_NORMAL_TURN);
            UNO.ActionManager.Instance.updateSelectPlayer(player);
            UNO.GameRuleController.Instance.UpdatePlayerOperation(UNO.Player.Operaton.SkipTurn, player.Index);

            if (eventCall) {
                player.onStartTurn -= skipEvents[player.Index];
            }
        }

        public static bool ShowCards() {
            if (UNO.CheatManager.Instance.ShowCards) {
                return false;
            }

            UNO.CheatManager.Instance.ShowAICards();
            UNO.CheatManager.Instance.ShowCards = true;

            System.Timers.Timer timer = new System.Timers.Timer(); //new Timer(1000);

            timer.Elapsed += (sender, e) =>
            {
                UNO.CheatManager.Instance.HideAICards();
                UNO.CheatManager.Instance.ShowCards = false;
                timer.Stop();
            };
            timer.Interval = 5000;//miliseconds
            timer.Start();

            return true;
        }
    }
}
